#include "cellulamainwindow.h"
#include <QTime>
#include <QMenuBar>
#include <QMenu>
#include <QMdiSubWindow>
#include <QFileDialog>
#include "texteditor.h"

CellulaMainWindow::CellulaMainWindow()
{
    createAction();
    createMenu();
    createToolbar();
    translateGUI();

    option_dialog = new Options(this);
    setWindowIcon(QIcon(":/images/glide.svg"));

    mdi = new QMdiArea(this);

    //gl_widget = new GLwidget(QGLFormat(QGL::DoubleBuffer), this);
    //gl_widget->resize(0, 0);

    setCentralWidget(mdi);
    setGeometry(0,0, 800, 600);
    connect(actionOption, SIGNAL(activated()), this, SLOT(showOptionDialog()));
    connect(actionNew, SIGNAL(activated()), this, SLOT(newModel()));
    connect(actionRun, SIGNAL(activated()), this, SLOT(startSimulation()));
    connect(actionPause, SIGNAL(activated()), this, SLOT(stopSimulation()));
    connect(actionQuit, SIGNAL(activated()), this, SLOT(close()));
    connect(actionTile, SIGNAL(activated()), mdi, SLOT(tileSubWindows()));
    connect(actionCascade, SIGNAL(activated()), mdi, SLOT(cascadeSubWindows()));
    connect(actionOpen, SIGNAL(activated()), this, SLOT(load()));
    connect(actionSave, SIGNAL(activated()), this, SLOT(save()));

    machine = 0;
    createCLMachine();

    QString text = "[top]\n"
    "components : life\n"
    "[life]\n"
    "type : cell\n"
    "width : 20\n"
    "height : 20\n"
    "delay : transport\n"
    "border : wrapped\n"
    "neighbors : life(-1,-1) life(-1,0) life(-1,1)\n"
    "neighbors : life(0,-1) life(0,0) life(0,1)\n"
    "neighbors : life(1,-1) life(1,0) life(1,1)\n"
    "initialvalue : 0\n"
    "initialrowvalue : 1 00010001111000000000\n"
    "initialrowvalue : 2 00110111100010111100\n"
    "initialrowvalue : 3 00110000011110000010\n"
    "initialrowvalue : 4 00101111000111100011\n"
    "initialrowvalue : 10 01111000111100011110\n"
    "initialrowvalue : 11 00010001111000000000\n"
    "localtransition : life-rule\n"
    "\n"
    "[life-rule]\n"
    "rule : 1 100 { (0,0) = 1 and (truecount = 3 or truecount = 4) }\n"
    "rule : 1 100 { (0,0) = 0 and truecount = 2 }\n"
    "rule : 0 100 { t }\n";

    TextEditor *editor = new TextEditor(text, this);
    mdi->addSubWindow(editor);
    editor->setName("Live");
}

CellulaMainWindow::~CellulaMainWindow()
{
    //delete model;
    delete machine;
}

void CellulaMainWindow::createMenu()
{
    QMenuBar *menuBar = new QMenuBar(this);
    QMenu *file_menu = new QMenu(trUtf8("Súbor"), menuBar);
    file_menu->addAction(actionNew);
    file_menu->addAction(actionOpen);
    //file_menu->addAction(actionSave);
    file_menu->addSeparator();
    file_menu->addAction(actionOption);
    file_menu->addAction(actionQuit);

    QMenu *run_menu = new QMenu(trUtf8("Beh"), menuBar);
    run_menu->addAction(actionRun);
    run_menu->addAction(actionPause);
    run_menu->addAction(actionStop);

    QMenu *window_menu = new QMenu(trUtf8("Okno"), menuBar);
    window_menu->addAction(actionTile);
    window_menu->addAction(actionCascade);

    menuBar->addMenu(file_menu);
    menuBar->addMenu(run_menu);
    menuBar->addMenu(window_menu);
    setMenuBar(menuBar);
}

void CellulaMainWindow::createToolbar()
{
    main_toolbar = new QToolBar(trUtf8("Hlavný panel nástrojov"), this);
    main_toolbar->addAction(actionNew);
    main_toolbar->addAction(actionOpen);
    main_toolbar->addAction(actionSave);
    main_toolbar->addAction(actionRun);
    main_toolbar->addAction(actionPause);
    main_toolbar->addAction(actionStop);
    addToolBar(main_toolbar);
}

void CellulaMainWindow::translateGUI()
{
    setWindowTitle(trUtf8("Cellula"));
}

void CellulaMainWindow::createAction()
{
    QStyle *cur_style = QApplication::style();
    actionNew = new QAction(trUtf8("Nový model"), this);
    actionNew->setIcon(cur_style->standardIcon(QStyle::SP_FileIcon));
    actionOpen = new QAction(trUtf8("Otvoriť model"), this);
    actionOpen->setIcon(cur_style->standardIcon(QStyle::SP_DirIcon));
    actionSave = new QAction(trUtf8("Ulož model"), this);
    actionSave->setIcon(cur_style->standardIcon(QStyle::SP_DriveFDIcon));
    actionRun = new QAction(trUtf8("Spusti simuláciu"), this);
    actionRun->setIcon(cur_style->standardIcon(QStyle::SP_MediaPlay));
    actionPause = new QAction(trUtf8("Pozastav simuláciu"), this);
    actionPause->setIcon(cur_style->standardIcon(QStyle::SP_MediaPause));
    actionStop = new QAction(trUtf8("Zastav simuláciu"), this);
    actionStop->setIcon(cur_style->standardIcon(QStyle::SP_MediaStop));
    actionOption = new QAction(trUtf8("Nastavenia"), this);
    actionQuit = new QAction(trUtf8("Koniec"), this);
    actionTile = new QAction(trUtf8("Dlaždice"), this);
    actionCascade = new QAction(trUtf8("Kaskáda"), this);
}

void CellulaMainWindow::showOptionDialog()
{
    option_dialog->show();
}

void CellulaMainWindow::createCLMachine()
{
    //delete machine;
    //gl_widget->makeCurrent();
    //gl_widget->updateGL();
    //machine = new Machine((cl_context_properties)glXGetCurrentContext(), (cl_context_properties)glXGetCurrentDisplay(), option_dialog->getPlatform(), option_dialog->getDevices());
    machine = new Machine(option_dialog->getPlatform(), option_dialog->getDevices());
    model = new SynchronousModel(machine, "");

    /*unsigned char *tex = space->renderToTexture();
    gl_widget->loadTexture(space->getWidth(), space->getHeight(), tex);
    delete [] tex;*/
    asyn_model = new AsynchronousModel(machine, "");
    GLwidget *asyn_widget = new GLwidget(QGLFormat(QGL::DoubleBuffer), this, new CellSpace(asyn_model, 16, 16));
    float asyn_data[] = {1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                         0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
    ((AsynchronousModel*)asyn_model)->initialize(asyn_widget->getSpace(), asyn_data);
    QMdiSubWindow *subwindow = mdi->addSubWindow(asyn_widget);
    subwindow->show();
    asyn_widget->setSubWindowTitle();
}

void CellulaMainWindow::startSimulation()
{
    GLwidget *active_window = dynamic_cast<GLwidget*>(mdi->activeSubWindow() ? mdi->activeSubWindow()->widget() : 0);
    if(active_window)
        active_window->startSimulation();
}

void CellulaMainWindow::stopSimulation()
{
    GLwidget *active_window = dynamic_cast<GLwidget*>(mdi->activeSubWindow() ? mdi->activeSubWindow()->widget() : 0);
    if(active_window)
        active_window->stopSimulation();
}

void CellulaMainWindow::newModel()
{
    GLwidget *new_glwidget = new GLwidget(QGLFormat(QGL::DoubleBuffer), this, new CellSpace(model, 256, 256));
    QMdiSubWindow *subwindow = mdi->addSubWindow(new_glwidget);
    subwindow->show();
    new_glwidget->setSubWindowTitle();
}

void CellulaMainWindow::load()
{
    QSettings setting;
    QString dir = setting.value("file/recent_dir", QDir::homePath()).toString();
    QString file = QFileDialog::getOpenFileName(this, trUtf8("Otvoriť súbor"), dir, trUtf8("Definícia Cellula modelu alebo stavu (*.xml)"));
    if(file.isNull())return;
    QFileInfo file_info(file);
    QDir recent_dir = file_info.absoluteDir();
    cout << recent_dir.absolutePath().toStdString() << endl;
    setting.setValue("file/recent_dir", recent_dir.absolutePath());
}

void CellulaMainWindow::save()
{
    QSettings setting;
    QString dir = setting.value("file/recent_dir", QDir::homePath()).toString();
    QMdiSubWindow *subWindow = mdi->activeSubWindow();
    if(subWindow)
    {
        SubWindowInterface *interface = dynamic_cast<SubWindowInterface*>(subWindow->widget());
        if(interface)
        {
            dir = dir+"/"+interface->getFileName()+".xml";

            QString file = QFileDialog::getSaveFileName(this, trUtf8("Otvoriť súbor"), dir, trUtf8("Definícia Cellula modelu alebo stavu (*.xml)"));
            if(file.isNull())return;
            QFileInfo file_info(file);
            QDir recent_dir = file_info.absoluteDir();
            setting.setValue("file/recent_dir", recent_dir.absolutePath());
        }
    }
}
